## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----echo=FALSE---------------------------------------------------------------
library(knitr)
Sys.setenv(LANGUAGE = "en")

## ----plot, echo=FALSE, fig.cap="PC1_discordance_cases.png", out.width = '50%'----
knitr::include_graphics("./Plots/PC1_discordance_cases.png")

## ----hmap1, echo=FALSE, fig.cap="PC1ihc.Mdns_PAM50_normalized_heatmap.pdf", out.width = '70%'----
knitr::include_graphics("./Plots/PC1ihc.Mdns_PAM50_normalized_heatmap.pdf")

## ----hmap2, echo=FALSE, fig.cap="PCAPAM50.Mdns_PAM50_normalized_heatmap.pdf", out.width = '70%'----
knitr::include_graphics("./Plots/PCAPAM50.Mdns_PAM50_normalized_heatmap.pdf")

## ----hmap3, echo=FALSE, fig.cap="Ihc.Mdns_PAM50_normalized_heatmap.pdf", out.width = '70%'----
knitr::include_graphics("./Plots/Ihc.Mdns_PAM50_normalized_heatmap.pdf")

